﻿using System;
using System.Collections.Generic;
using System.Text;

namespace SpringbearWorks.WinForms.Lcd
{
    public sealed class CharacterGenerator
    {
        #region DDRAM
        /// <summary>
        /// Get character data from DDRAM by address.
        /// </summary>
        /// <param name="address">DDRAM address.</param>
        /// <returns>8 bytes character data.</returns>
        public static byte[] GetDdram(byte address)
        {
            return charset[address];
        }
        /// <summary>
        /// Get character data from DDRAM to match the given character.
        /// </summary>
        /// <param name="character">Character to display (ASCII only).</param>
        /// <returns>8 bytes character data.</returns>
        public static byte[] GetDdram(char character)
        {
            return charset[(byte)character];
        }
        #endregion

        #region CGRAM
        // 8 cgram chars
        static byte[][] cgram = new byte[8][];
        // for dummy
        static byte[] emptyChar ={ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
        /// <summary>
        /// Store character data in CGRAM registers by index.
        /// </summary>
        /// <param name="data">Data to be stored.</param>
        /// <param name="index">Index of register.</param>
        public static void SetCgram(byte[] data, int index)
        {
            if (data == null || data.Length != 8)
                return;
            if (index < 0 || index > 7)
                return;

            cgram[index] = data;
        }
        /// <summary>
        /// Get CGRAM character data by index.
        /// </summary>
        /// <param name="index">Index of register.</param>
        /// <returns>Character data.</returns>
        public static byte[] GetCgram(int index)
        {
            if (index < 0 || index > 7)
                return emptyChar;

            return cgram[index];
        }
        #endregion

        #region Charset
        // 256x8 bytes (1024 bytes)
        static readonly byte[][] charset =
            {
#region 0000
                // 0000 0000
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 0001
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 0010
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 0011
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 0100
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 0101
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 0110
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 0111
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 1000
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 1001
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 1010
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 1011
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 1100
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 1101
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 1110
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0000 1111
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                //=========================================
#endregion
                
#region 0001
                // 0001 0000
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 0001
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 0010
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 0011
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 0100
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 0101
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 0110
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 0111
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 1000
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 1001
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 1010
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 1011
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 1100
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 1101
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 1110
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0001 1111
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                //=========================================
#endregion

#region 0010
                // 0010 0000
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0010 0001    !
                new byte[]{ 0x04, 0x04, 0x04, 0x04, 0x00, 0x00, 0x04, 0x00 },
                // 0010 0010    "
                new byte[]{ 0x0a, 0x0a, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0010 0011    #
                new byte[]{ 0x0a, 0x0a, 0x1f, 0x0a, 0x1f, 0x0a, 0x0a, 0x00 },
                // 0010 0100    $
                new byte[]{ 0x04, 0x0f, 0x14, 0x0e, 0x05, 0x1e, 0x04, 0x00 },
                // 0010 0101    %
                new byte[]{ 0x18, 0x19, 0x02, 0x04, 0x08, 0x13, 0x03, 0x00 },
                // 0010 0110    &
                new byte[]{ 0x0c, 0x12, 0x14, 0x08, 0x15, 0x12, 0x0d, 0x00 },
                // 0010 0111    '
                new byte[]{ 0x0c, 0x04, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0010 1000    (
                new byte[]{ 0x02, 0x04, 0x08, 0x08, 0x08, 0x04, 0x02, 0x00 },
                // 0010 1001    )
                new byte[]{ 0x08, 0x04, 0x02, 0x02, 0x02, 0x04, 0x08, 0x00 },
                // 0010 1010    *
                new byte[]{ 0x00, 0x04, 0x15, 0x0e, 0x15, 0x04, 0x00, 0x00 },
                // 0010 1011    +
                new byte[]{ 0x00, 0x04, 0x04, 0x1f, 0x04, 0x04, 0x00, 0x00 },
                // 0010 1100    ,
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x0c, 0x04, 0x08, 0x00 },
                // 0010 1101    -
                new byte[]{ 0x00, 0x00, 0x00, 0x1f, 0x00, 0x00, 0x00, 0x00 },
                // 0010 1110    .
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x0c, 0x00 },
                // 0010 1111    /
                new byte[]{ 0x00, 0x01, 0x02, 0x04, 0x08, 0x10, 0x00, 0x00 },
                //=========================================
#endregion

#region 0011
                // 0011 0000    0
                new byte[]{ 0x0e, 0x11, 0x13, 0x15, 0x19, 0x11, 0x0e, 0x00 },
                // 0011 0001    1
                new byte[]{ 0x04, 0x0c, 0x04, 0x04, 0x04, 0x04, 0x0e, 0x00 },
                // 0011 0010    2
                new byte[]{ 0x0e, 0x11, 0x01, 0x02, 0x04, 0x08, 0x1f, 0x00 },
                // 0011 0011    3
                new byte[]{ 0x1f, 0x02, 0x04, 0x02, 0x01, 0x11, 0x0e, 0x00 },
                // 0011 0100    4
                new byte[]{ 0x02, 0x06, 0x0a, 0x12, 0x1f, 0x02, 0x02, 0x00 },
                // 0011 0101    5
                new byte[]{ 0x1f, 0x10, 0x1e, 0x01, 0x01, 0x11, 0x0e, 0x00 },
                // 0011 0110    6
                new byte[]{ 0x06, 0x08, 0x10, 0x1e, 0x11, 0x11, 0x0e, 0x00 },
                // 0011 0111    7
                new byte[]{ 0x1f, 0x11, 0x02, 0x04, 0x04, 0x04, 0x04, 0x00 },
                // 0011 1000    8
                new byte[]{ 0x0e, 0x11, 0x11, 0x0e, 0x11, 0x11, 0x0e, 0x00 },
                // 0011 1001    9
                new byte[]{ 0x0e, 0x11, 0x11, 0x0f, 0x01, 0x11, 0x0e, 0x00 },
                // 0011 1010    :
                new byte[]{ 0x00, 0x0c, 0x0c, 0x00, 0x0c, 0x0c, 0x00, 0x00 },
                // 0011 1011    ;
                new byte[]{ 0x00, 0x0c, 0x0c, 0x00, 0x0c, 0x04, 0x08, 0x00 },
                // 0011 1100    <
                new byte[]{ 0x02, 0x04, 0x08, 0x10, 0x08, 0x04, 0x02, 0x00 },
                // 0011 1101    =
                new byte[]{ 0x00, 0x00, 0x1f, 0x00, 0x1f, 0x00, 0x00, 0x00 },
                // 0011 1110    >
                new byte[]{ 0x10, 0x08, 0x04, 0x02, 0x04, 0x08, 0x10, 0x00 },
                // 0011 1111    ?
                new byte[]{ 0x0e, 0x11, 0x01, 0x02, 0x04, 0x00, 0x04, 0x00 },
                //=========================================
#endregion

#region 0100
                // 0100 0000    @
                new byte[]{ 0x0e, 0x11, 0x01, 0x0d, 0x15, 0x15, 0x0e, 0x00 },
                // 0100 0001    A
                new byte[]{ 0x0e, 0x11, 0x11, 0x11, 0x1f, 0x11, 0x11, 0x00 },
                // 0100 0010    B
                new byte[]{ 0x1e, 0x11, 0x11, 0x1e, 0x11, 0x11, 0x1e, 0x00 },
                // 0100 0011    C
                new byte[]{ 0x0e, 0x11, 0x10, 0x10, 0x10, 0x11, 0x0e, 0x00 },
                // 0100 0100    D
                new byte[]{ 0x1e, 0x11, 0x11, 0x11, 0x11, 0x11, 0x1e, 0x00 },
                // 0100 0101    E
                new byte[]{ 0x1f, 0x10, 0x10, 0x1e, 0x10, 0x10, 0x1f, 0x00 },
                // 0100 0110    F
                new byte[]{ 0x1f, 0x10, 0x10, 0x1e, 0x10, 0x10, 0x10, 0x00 },
                // 0100 0111    G
                new byte[]{ 0x0e, 0x11, 0x10, 0x17, 0x11, 0x11, 0x0e, 0x00 },
                // 0100 1000    H
                new byte[]{ 0x11, 0x11, 0x11, 0x1f, 0x11, 0x11, 0x11, 0x00 },
                // 0100 1001    I
                new byte[]{ 0x0e, 0x04, 0x04, 0x04, 0x04, 0x04, 0x0e, 0x00 },
                // 0100 1010    J
                new byte[]{ 0x07, 0x02, 0x02, 0x02, 0x02, 0x12, 0x0c, 0x00 },
                // 0100 1011    K
                new byte[]{ 0x11, 0x12, 0x14, 0x18, 0x14, 0x12, 0x11, 0x00 },
                // 0100 1100    L
                new byte[]{ 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x1f, 0x00 },
                // 0100 1101    M
                new byte[]{ 0x11, 0x1b, 0x15, 0x15, 0x11, 0x11, 0x11, 0x00 },
                // 0100 1110    N
                new byte[]{ 0x11, 0x11, 0x19, 0x15, 0x13, 0x11, 0x11, 0x00 },
                // 0100 1111    O
                new byte[]{ 0x0e, 0x11, 0x11, 0x11, 0x11, 0x11, 0x0e, 0x00 },
                //=========================================
#endregion

#region 0101
                // 0101 0000    P
                new byte[]{ 0x1e, 0x11, 0x11, 0x1e, 0x10, 0x10, 0x10, 0x00 },
                // 0101 0001    Q
                new byte[]{ 0x0e, 0x11, 0x11, 0x11, 0x15, 0x12, 0x0d, 0x00 },
                // 0101 0010    R
                new byte[]{ 0x1e, 0x11, 0x11, 0x1e, 0x14, 0x12, 0x11, 0x00 },
                // 0101 0011    S
                new byte[]{ 0x0f, 0x10, 0x10, 0x0e, 0x01, 0x01, 0x1e, 0x00 },
                // 0101 0100    T
                new byte[]{ 0x1f, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x00 },
                // 0101 0101    U
                new byte[]{ 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x0e, 0x00 },
                // 0101 0110    V
                new byte[]{ 0x11, 0x11, 0x11, 0x11, 0x11, 0x0a, 0x04, 0x00 },
                // 0101 0111    W
                new byte[]{ 0x11, 0x11, 0x11, 0x15, 0x15, 0x15, 0x0a, 0x00 },
                // 0101 1000    X
                new byte[]{ 0x11, 0x11, 0x0a, 0x04, 0x0a, 0x11, 0x11, 0x00 },
                // 0101 1001    Y
                new byte[]{ 0x11, 0x11, 0x11, 0x0a, 0x04, 0x04, 0x04, 0x00 },
                // 0101 1010    Z
                new byte[]{ 0x1f, 0x01, 0x02, 0x04, 0x08, 0x10, 0x1f, 0x00 },
                // 0101 1011    [
                new byte[]{ 0x0e, 0x08, 0x08, 0x08, 0x08, 0x08, 0x0e, 0x00 },
                // 0101 1100    ￥
                new byte[]{ 0x11, 0x0a, 0x1f, 0x04, 0x1f, 0x04, 0x04, 0x00 },
                // 0101 1101    ]
                new byte[]{ 0x0e, 0x02, 0x02, 0x02, 0x02, 0x02, 0x0e, 0x00 },
                // 0101 1110    ^
                new byte[]{ 0x04, 0x0a, 0x11, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0101 1111    _
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f, 0x00 },
                //=========================================
#endregion

#region 0110
                // 0110 0000    `
                new byte[]{ 0x08, 0x04, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 0110 0001    a
                new byte[]{ 0x00, 0x00, 0x0e, 0x01, 0x0f, 0x11, 0x0f, 0x00 },
                // 0110 0010    b
                new byte[]{ 0x10, 0x10, 0x10, 0x16, 0x19, 0x11, 0x1e, 0x00 },
                // 0110 0011    c
                new byte[]{ 0x00, 0x00, 0x0e, 0x10, 0x10, 0x11, 0x0e, 0x00 },
                // 0110 0100    d
                new byte[]{ 0x01, 0x01, 0x0d, 0x13, 0x11, 0x11, 0x0f, 0x00 },
                // 0110 0101    e
                new byte[]{ 0x00, 0x00, 0x0e, 0x11, 0x1f, 0x10, 0x0e, 0x00 },
                // 0110 0110    f
                new byte[]{ 0x06, 0x09, 0x08, 0x1c, 0x08, 0x08, 0x08, 0x00 },
                // 0110 0111    g
                new byte[]{ 0x00, 0x0f, 0x11, 0x11, 0x0f, 0x01, 0x0e, 0x00 },
                // 0110 1000    h
                new byte[]{ 0x10, 0x10, 0x10, 0x16, 0x19, 0x11, 0x11, 0x00 },
                // 0110 1001    i
                new byte[]{ 0x04, 0x00, 0x0c, 0x04, 0x04, 0x04, 0x0e, 0x00 },
                // 0110 1010    j
                new byte[]{ 0x02, 0x00, 0x06, 0x02, 0x02, 0x12, 0x0c, 0x00 },
                // 0110 1011    k
                new byte[]{ 0x10, 0x10, 0x12, 0x14, 0x18, 0x14, 0x12, 0x00 },
                // 0110 1100    l
                new byte[]{ 0x0c, 0x04, 0x04, 0x04, 0x04, 0x04, 0x0e, 0x00 },
                // 0110 1101    m
                new byte[]{ 0x00, 0x00, 0x1a, 0x15, 0x15, 0x11, 0x11, 0x00 },
                // 0110 1110    n
                new byte[]{ 0x00, 0x00, 0x16, 0x19, 0x11, 0x11, 0x11, 0x00 },
                // 0110 1111    o
                new byte[]{ 0x00, 0x00, 0x0e, 0x11, 0x11, 0x11, 0x0e, 0x00 },
                //=========================================
#endregion

#region 0111
                // 0111 0000    p
                new byte[]{ 0x00, 0x00, 0x1e, 0x11, 0x1e, 0x10, 0x10, 0x00 },
                // 0111 0001    q
                new byte[]{ 0x00, 0x00, 0x0d, 0x13, 0x0f, 0x01, 0x01, 0x00 },
                // 0111 0010    r
                new byte[]{ 0x00, 0x00, 0x16, 0x19, 0x10, 0x10, 0x10, 0x00 },
                // 0111 0011    s
                new byte[]{ 0x00, 0x00, 0x0e, 0x10, 0x0e, 0x01, 0x1e, 0x00 },
                // 0111 0100    t
                new byte[]{ 0x08, 0x08, 0x1c, 0x08, 0x08, 0x09, 0x06, 0x00 },
                // 0111 0101    u
                new byte[]{ 0x00, 0x00, 0x11, 0x11, 0x11, 0x13, 0x0d, 0x00 },
                // 0111 0110    v
                new byte[]{ 0x00, 0x00, 0x11, 0x11, 0x11, 0x0a, 0x04, 0x00 },
                // 0111 0111    w
                new byte[]{ 0x00, 0x00, 0x11, 0x15, 0x15, 0x15, 0x0a, 0x00 },
                // 0111 1000    x
                new byte[]{ 0x00, 0x00, 0x11, 0x0a, 0x04, 0x0a, 0x11, 0x00 },
                // 0111 1001    y
                new byte[]{ 0x00, 0x00, 0x11, 0x11, 0x0f, 0x01, 0x0e, 0x00 },
                // 0111 1010    z
                new byte[]{ 0x00, 0x00, 0x1f, 0x02, 0x04, 0x08, 0x1f, 0x00 },
                // 0111 1011    {
                new byte[]{ 0x02, 0x04, 0x04, 0x08, 0x04, 0x04, 0x02, 0x00 },
                // 0111 1100    |
                new byte[]{ 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x00 },
                // 0111 1101    }
                new byte[]{ 0x08, 0x04, 0x04, 0x02, 0x04, 0x04, 0x08, 0x00 },
                // 0111 1110    →
                new byte[]{ 0x00, 0x04, 0x02, 0x1f, 0x02, 0x04, 0x00, 0x00 },
                // 0111 1111    ←
                new byte[]{ 0x00, 0x04, 0x08, 0x1f, 0x08, 0x04, 0x00, 0x00 },
                //=========================================
#endregion

#region 1000 (custom chinese numbers)
                // 1000 0000    一
                new byte[]{ 0x00, 0x00, 0x00, 0x1f, 0x00, 0x00, 0x00, 0x00 },
                // 1000 0001    二
                new byte[]{ 0x00, 0x00, 0x0e, 0x00, 0x00, 0x1f, 0x00, 0x00 },
                // 1000 0010    三
                new byte[]{ 0x00, 0x0e, 0x00, 0x0e, 0x00, 0x1f, 0x00, 0x00 },
                // 1000 0011    四
                new byte[]{ 0x00, 0x1f, 0x15, 0x1b, 0x11, 0x1f, 0x00, 0x00 },
                // 1000 0100    五
                new byte[]{ 0x00, 0x1e, 0x08, 0x1e, 0x0a, 0x1f, 0x00, 0x00 },
                // 1000 0101    六
                new byte[]{ 0x00, 0x00, 0x04, 0x1f, 0x0a, 0x11, 0x00, 0x00 },
                // 1000 0110    七
                new byte[]{ 0x00, 0x08, 0x08, 0x1f, 0x08, 0x0e, 0x00, 0x00 },
                // 1000 0111    八
                new byte[]{ 0x00, 0x02, 0x0a, 0x0a, 0x0a, 0x11, 0x00, 0x00 },
                // 1000 1000    九
                new byte[]{ 0x00, 0x08, 0x1e, 0x0a, 0x0a, 0x13, 0x00, 0x00 },
                // 1000 1001    零
                new byte[]{ 0x0e, 0x11, 0x11, 0x11, 0x11, 0x11, 0x0e, 0x00 },
                // 1000 1010    十
                new byte[]{ 0x04, 0x04, 0x1f, 0x04, 0x04, 0x04, 0x00, 0x00 },
                // 1000 1011    百
                new byte[]{ 0x1f, 0x04, 0x1f, 0x11, 0x1f, 0x11, 0x1f, 0x00 },
                // 1000 1100    千
                new byte[]{ 0x02, 0x1c, 0x04, 0x1f, 0x04, 0x04, 0x04, 0x00 },
                // 1000 1101    万
                new byte[]{ 0x00, 0x1e, 0x08, 0x0f, 0x09, 0x09, 0x12, 0x00 },
                // 1000 1110    亿
                new byte[]{ 0x08, 0x17, 0x11, 0x12, 0x14, 0x15, 0x17, 0x00 },
                // 1000 1111    兆
                new byte[]{ 0x00, 0x0a, 0x1b, 0x0a, 0x1b, 0x0a, 0x13, 0x00 },
                //=========================================
#endregion

#region 1001 (custom chinese symbols)
                // 1001 0000    甲
                new byte[]{ 0x1f, 0x15, 0x1f, 0x15, 0x1f, 0x04, 0x04, 0x00 },
                // 1001 0001    乙
                new byte[]{ 0x1f, 0x02, 0x04, 0x08, 0x10, 0x11, 0x0f, 0x00 },
                // 1001 0010    丙
                new byte[]{ 0x1f, 0x04, 0x1f, 0x15, 0x1b, 0x11, 0x13, 0x00 },
                // 1001 0011    丁
                new byte[]{ 0x1f, 0x04, 0x04, 0x04, 0x04, 0x04, 0x0c, 0x00 },
                // 1001 0100    电池-空
                new byte[]{ 0x0e, 0x11, 0x11, 0x11, 0x11, 0x11, 0x1f, 0x00 },
                // 1001 0101    电池-20%
                new byte[]{ 0x0e, 0x11, 0x11, 0x11, 0x11, 0x1f, 0x1f, 0x00 },
                // 1001 0110    电池-40%
                new byte[]{ 0x0e, 0x11, 0x11, 0x11, 0x1f, 0x1f, 0x1f, 0x00 },
                // 1001 0111    电池-60%
                new byte[]{ 0x0e, 0x11, 0x11, 0x1f, 0x1f, 0x1f, 0x1f, 0x00 },
                // 1001 1000    电池-80%
                new byte[]{ 0x0e, 0x11, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x00 },
                // 1001 1001    电池-满
                new byte[]{ 0x0e, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x00 },
                // 1001 1010    钥匙
                new byte[]{ 0x0e, 0x11, 0x0e, 0x04, 0x0c, 0x04, 0x0c, 0x00 },
                // 1001 1011    酒杯
                new byte[]{ 0x1f, 0x11, 0x11, 0x0e, 0x04, 0x04, 0x0e, 0x00 },
                // 1001 1100    T/R 正常
                new byte[]{ 0x00, 0x0e, 0x04, 0x04, 0x00, 0x0a, 0x0c, 0x08 },
                // 1001 1101    T/R 反白
                new byte[]{ 0x1f, 0x11, 0x1b, 0x1b, 0x1f, 0x15, 0x13, 0x17 },
                // 1001 1110    T/R - T 反白
                new byte[]{ 0x1f, 0x11, 0x1b, 0x1b, 0x00, 0x0a, 0x0c, 0x08 },
                // 1001 1111    T/R - R 反白
                new byte[]{ 0x00, 0x0e, 0x04, 0x04, 0x1f, 0x15, 0x13, 0x17 },
                //=========================================
#endregion

#region 1010 (japanese & symbols)
                // 1010 0000
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1010 0001    。
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x1c, 0x14, 0x1c, 0x00 },
                // 1010 0010    「
                new byte[]{ 0x07, 0x04, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1010 0011    」
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x04, 0x04, 0x1c, 0x00 },
                // 1010 0100    、
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x10, 0x08, 0x04, 0x00 },
                // 1010 0101    ·
                new byte[]{ 0x00, 0x00, 0x0c, 0x0c, 0x00, 0x00, 0x00, 0x00 },
                // 1010 0110    
                new byte[]{ 0x00, 0x1f, 0x01, 0x1f, 0x01, 0x02, 0x04, 0x00 },
                // 1010 0111
                new byte[]{ 0x00, 0x00, 0x1f, 0x01, 0x06, 0x04, 0x08, 0x00 },
                // 1010 1000
                new byte[]{ 0x00, 0x00, 0x02, 0x04, 0x0c, 0x14, 0x04, 0x00 },
                // 1010 1001
                new byte[]{ 0x00, 0x00, 0x04, 0x1f, 0x11, 0x01, 0x06, 0x00 },
                // 1010 1010
                new byte[]{ 0x00, 0x00, 0x1f, 0x04, 0x04, 0x04, 0x1f, 0x00 },
                // 1010 1011
                new byte[]{ 0x00, 0x00, 0x02, 0x1f, 0x06, 0x0a, 0x12, 0x00 },
                // 1010 1100
                new byte[]{ 0x00, 0x00, 0x08, 0x1f, 0x09, 0x0a, 0x08, 0x00 },
                // 1010 1101
                new byte[]{ 0x00, 0x00, 0x00, 0x0e, 0x02, 0x02, 0x1f, 0x00 },
                // 1010 1110
                new byte[]{ 0x00, 0x00, 0x1e, 0x02, 0x1e, 0x02, 0x1e, 0x00 },
                // 1010 1111
                new byte[]{ 0x00, 0x00, 0x00, 0x15, 0x15, 0x01, 0x06, 0x00 },
                //=========================================
#endregion

#region 1011
                // 1011 0000
                new byte[]{ 0x00, 0x00, 0x1e, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1011 0001
                new byte[]{ 0x1f, 0x01, 0x05, 0x06, 0x04, 0x04, 0x08, 0x00 },
                // 1011 0010
                new byte[]{ 0x01, 0x02, 0x04, 0x0c, 0x14, 0x04, 0x04, 0x00 },
                // 1011 0011
                new byte[]{ 0x04, 0x1f, 0x11, 0x11, 0x01, 0x02, 0x04, 0x00 },
                // 1011 0100
                new byte[]{ 0x1f, 0x04, 0x04, 0x04, 0x04, 0x04, 0x1f, 0x00 },
                // 1011 0101
                new byte[]{ 0x02, 0x1f, 0x02, 0x06, 0x0a, 0x12, 0x02, 0x00 },
                // 1011 0110
                new byte[]{ 0x08, 0x1f, 0x09, 0x09, 0x09, 0x09, 0x12, 0x00 },
                // 1011 0111
                new byte[]{ 0x04, 0x1f, 0x04, 0x1f, 0x04, 0x04, 0x04, 0x00 },
                // 1011 1000
                new byte[]{ 0x00, 0x0f, 0x09, 0x11, 0x01, 0x02, 0x0c, 0x00 },
                // 1011 1001
                new byte[]{ 0x08, 0x0f, 0x12, 0x02, 0x02, 0x02, 0x04, 0x00 },
                // 1011 1010
                new byte[]{ 0x1f, 0x01, 0x01, 0x01, 0x01, 0x01, 0x1f, 0x00 },
                // 1011 1011
                new byte[]{ 0x0a, 0x1f, 0x0a, 0x0a, 0x02, 0x04, 0x08, 0x00 },
                // 1011 1100
                new byte[]{ 0x18, 0x01, 0x19, 0x01, 0x01, 0x02, 0x1c, 0x00 },
                // 1011 1101
                new byte[]{ 0x00, 0x1f, 0x01, 0x02, 0x04, 0x0a, 0x11, 0x00 },
                // 1011 1110
                new byte[]{ 0x08, 0x1f, 0x09, 0x0a, 0x08, 0x08, 0x07, 0x00 },
                // 1011 1111
                new byte[]{ 0x00, 0x11, 0x11, 0x09, 0x01, 0x02, 0x0c, 0x00 },
                //=========================================
#endregion

#region 1100
                // 1100 0000
                new byte[]{ 0x00, 0x0f, 0x09, 0x15, 0x02, 0x02, 0x0c, 0x00 },
                // 1100 0001
                new byte[]{ 0x02, 0x1c, 0x04, 0x1f, 0x04, 0x04, 0x08, 0x00 },
                // 1100 0010
                new byte[]{ 0x15, 0x15, 0x15, 0x01, 0x01, 0x02, 0x04, 0x00 },
                // 1100 0011
                new byte[]{ 0x0e, 0x00, 0x1f, 0x04, 0x04, 0x04, 0x08, 0x00 },
                // 1100 0100
                new byte[]{ 0x08, 0x08, 0x08, 0x0c, 0x0a, 0x08, 0x08, 0x00 },
                // 1100 0101
                new byte[]{ 0x04, 0x04, 0x1f, 0x04, 0x04, 0x08, 0x10, 0x00 },
                // 1100 0110
                new byte[]{ 0x00, 0x0e, 0x00, 0x00, 0x00, 0x00, 0x1f, 0x00 },
                // 1100 0111
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1100 1000
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1100 1001
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1100 1010
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1100 1011
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1100 1100
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1100 1101
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1100 1110
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1100 1111
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                //=========================================
#endregion

#region 1101
                // 1101 0000
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 0001
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 0010
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 0011
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 0100
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 0101
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 0110
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 0111
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 1000
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 1001
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 1010
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 1011
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 1100
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 1101
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 1110
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1101 1111
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                //=========================================
#endregion

#region 1110
                // 1110 0000
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 0001
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 0010
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 0011
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 0100
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 0101
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 0110
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 0111
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 1000
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 1001
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 1010
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 1011
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 1100
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 1101
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 1110
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1110 1111
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                //=========================================
#endregion

#region 1111
                // 1111 0000
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 0001
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 0010
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 0011
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 0100
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 0101
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 0110
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 0111
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 1000
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 1001
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 1010
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 1011
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 1100
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 1101
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 1110
                new byte[]{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                // 1111 1111
                new byte[]{ 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f }
            };
#endregion
        #endregion
    }
}
